@@ -22,18 +22,18 @@ namespace xmlpp
 {
 
 DtdValidator::DtdValidator()
-: dtd_(0)
+: dtd_(nullptr)
 {
 }
 
 DtdValidator::DtdValidator(const Glib::ustring& file)
-: dtd_(0)
+: dtd_(nullptr)
 {
   parse_subset("",file);
 }
 
 DtdValidator::DtdValidator(const Glib::ustring& external,const Glib::ustring& system)
-: dtd_(0)
+: dtd_(nullptr)
 {
   parse_subset(external,system);
 }
@@ -54,9 +54,9 @@ void DtdValidator::parse_subset(const Glib::ustring& external,const Glib::ustrin
   release_underlying(); // Free any existing dtd.
   xmlResetLastError();
 
-  xmlDtd* dtd = xmlParseDTD(
-    external.empty() ? 0 : (const xmlChar *)external.c_str(),
-    system.empty() ? 0 : (const xmlChar *)system.c_str());
+  auto dtd = xmlParseDTD(
+    external.empty() ? nullptr : (const xmlChar *)external.c_str(),
+    system.empty() ? nullptr : (const xmlChar *)system.c_str());
 
   if (!dtd)
   {
@@ -82,7 +82,7 @@ void DtdValidator::parse_stream(std::istream& in)
 
   IStreamParserInputBuffer ibuff( in );
 
-  xmlDtd* dtd = xmlIOParseDTD( 0, ibuff.cobj(), XML_CHAR_ENCODING_UTF8 );
+  auto dtd = xmlIOParseDTD(nullptr, ibuff.cobj(), XML_CHAR_ENCODING_UTF8);
 
   if (!dtd)
   {
@@ -99,16 +99,16 @@ void DtdValidator::release_underlying()
   {
     //Make a local pointer to the underlying xmlDtd object as the wrapper is destroyed first.
     //After free_wrappers is called dtd_ will be invalid (e.g. delete dtd_)
-    xmlDtd* dtd = dtd_->cobj();
+    auto dtd = dtd_->cobj();
     Node::free_wrappers(reinterpret_cast<xmlNode*>(dtd));
     xmlFreeDtd(dtd);
-    dtd_ = 0;
+    dtd_ = nullptr;
   }
 }
 
 DtdValidator::operator bool() const
 {
-  return dtd_ != 0;
+  return dtd_ != nullptr;
 }
 
 Dtd* DtdValidator::get_dtd()
@@ -125,7 +125,7 @@ bool DtdValidator::validate(const Document* doc)
 {
   if (!doc)
   {
-    throw internal_error("Document pointer cannot be 0.");
+    throw internal_error("Document pointer cannot be nullptr.");
   }
 
   if (!dtd_)
