@@ -5,17 +5,18 @@
  * included with libxml++ as the file COPYING.
  */
 
+#include "libxml++/exceptions/wrapped_exception.h"
 #include "libxml++/validators/validator.h"
 
 #include <libxml/parser.h>
 
 #include <cstdarg> //For va_list.
-#include <memory> //For auto_ptr.
+#include <memory> //For unique_ptr.
 
 namespace xmlpp {
 
 Validator::Validator()
-: valid_(0), exception_(0)
+: valid_(nullptr), exception_(nullptr)
 {
 }
 
@@ -26,12 +27,17 @@ Validator::~Validator()
 
 void Validator::initialize_valid()
 {
-  //Tell the validation context about the callbacks:
-  valid_->error = &callback_validity_error;
-  valid_->warning = &callback_validity_warning;
+  // valid_ is used only by DtdValidator.
+  //TODO: When we can break ABI, move valid_ to DtdValidator.
+  if (valid_)
+  {
+    //Tell the validation context about the callbacks:
+    valid_->error = &callback_validity_error;
+    valid_->warning = &callback_validity_warning;
 
-  //Allow the callback_validity_*() methods to retrieve the C++ instance:
-  valid_->userData = this;
+    //Allow the callback_validity_*() methods to retrieve the C++ instance:
+    valid_->userData = this;
+  }
 
   //Clear these temporary buffers too:
   validate_error_.erase();
@@ -42,10 +48,10 @@ void Validator::release_underlying()
 {
   if(valid_)
   {
-    valid_->userData = 0; //Not really necessary.
+    valid_->userData = nullptr; //Not really necessary.
 
     xmlFreeValidCtxt(valid_);
-    valid_ = 0;
+    valid_ = nullptr;
   }
 }
 
@@ -89,7 +95,7 @@ void Validator::check_for_validity_messages()
 
 void Validator::callback_validity_error(void* valid_, const char* msg, ...)
 {
-  Validator* validator = static_cast<Validator*>(valid_);
+  auto validator = static_cast<Validator*>(valid_);
 
   if(validator)
   {
@@ -105,16 +111,16 @@ void Validator::callback_validity_error(void* valid_, const char* msg, ...)
     {
       validator->on_validity_error(Glib::ustring(buff));
     }
-    catch(const exception& e)
+    catch (...)
     {
-      validator->handleException(e);
+      validator->handle_exception();
     }
   }
 }
 
 void Validator::callback_validity_warning(void* valid_, const char* msg, ...)
 {
-  Validator* validator = static_cast<Validator*>(valid_);
+  auto validator = static_cast<Validator*>(valid_);
 
   if(validator)
   {
@@ -130,9 +136,9 @@ void Validator::callback_validity_warning(void* valid_, const char* msg, ...)
     {
       validator->on_validity_warning(Glib::ustring(buff));
     }
-    catch(const exception& e)
+    catch (...)
     {
-      validator->handleException(e);
+      validator->handle_exception();
     }
   }
 }
@@ -154,14 +160,56 @@ void Validator::handleException(const exception& e)
   //release_underlying();
 }
 
+void Validator::handle_exception()
+{
+  delete exception_;
+  exception_ = nullptr;
+
+  try
+  {
+    throw; // Rethrow current exception
+  }
+  catch (const exception& e)
+  {
+    exception_ = e.Clone();
+  }
+#ifdef LIBXMLXX_HAVE_EXCEPTION_PTR
+  catch (...)
+  {
+    exception_ = new wrapped_exception(std::current_exception());
+  }
+#else
+  catch (const std::exception& e)
+  {
+    exception_ = new exception(e.what());
+  }
+  catch (...)
+  {
+    exception_ = new exception("An exception was thrown that is not derived from std::exception or xmlpp::exception.\n"
+      "It could not be caught and rethrown because this platform does not support std::exception_ptr.");
+  }
+#endif
+
+  // Don't delete the DTD validation context or schema validation context
+  // while validating. It would cause accesses to deallocated memory in libxml2
+  // functions after the return from Validator::callback_validity_...().
+  // Parser::handle_exception() calls xmlStopParser(), but there is no
+  // xmlStopValidator() or similar function to call here.
+  // We don't throw the exception here, since it would have to pass through
+  // C functions. That's not guaranteed to work. It might work, but it depends
+  // on the C compiler and the options used when building libxml2.
+
+  //release_underlying();
+}
+
 void Validator::check_for_exception()
 {
   check_for_validity_messages();
 
   if(exception_)
   {
-    std::auto_ptr<exception> tmp(exception_);
-    exception_ = 0;
+    std::unique_ptr<exception> tmp(exception_);
+    exception_ = nullptr;
     tmp->Raise();
   }
 }
